var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
import { Component, Inject, trigger, style, state, transition, animate, ViewChildren } from '@angular/core';
import { FormControl } from '@angular/forms';
import { SANDBOXES } from './shared/tokens';
import 'rxjs/add/operator/debounceTime';
import 'rxjs/add/operator/distinctUntilChanged';
import { StateService } from './shared/state.service';
import { EventManager } from "@angular/platform-browser";
import { UrlService } from "./shared/url.service";
import { fuzzySearch } from './shared/fuzzy-search.function';
export var AppComponent = (function () {
    function AppComponent(sandboxes, stateService, urlService, eventManager) {
        var _this = this;
        this.stateService = stateService;
        this.urlService = urlService;
        this.eventManager = eventManager;
        this.commandBarActive = false;
        this.selectedSandboxAndScenarioKeys = { sandboxKey: null, scenarioKey: null };
        this.filter = new FormControl();
        if (this.urlService.embed) {
            this.selectedSandboxAndScenarioKeys = { sandboxKey: this.urlService.select.sandboxKey, scenarioKey: this.urlService.select.scenarioKey };
        }
        else {
            var filterValue = this.stateService.getFilter();
            if (this.urlService.select) {
                filterValue = (filterValue && fuzzySearch(filterValue.toLowerCase(), this.urlService.select.filter.toLowerCase())) ? filterValue : this.urlService.select.filter;
                this.selectedSandboxAndScenarioKeys = { sandboxKey: this.urlService.select.sandboxKey, scenarioKey: this.urlService.select.scenarioKey };
            }
            this.eventManager.addGlobalEventListener('window', 'keydown.control.o', function (e) {
                e.preventDefault();
            });
            this.eventManager.addGlobalEventListener('window', 'keyup.control.o', function () {
                _this.toggleCommandBar();
            });
            this.totalSandboxes = sandboxes.length;
            this.filteredSandboxes = this.filterSandboxes(sandboxes, filterValue);
            this.filter.setValue(filterValue);
            this.filter.valueChanges
                .debounceTime(300)
                .distinctUntilChanged()
                .subscribe(function (value) {
                _this.stateService.setFilter(value);
                _this.filteredSandboxes = _this.filterSandboxes(sandboxes, value);
                if (!value) {
                    _this.selectScenario(null, null);
                }
            });
        }
    }
    AppComponent.prototype.goToFirstScenario = function (event) {
        event.preventDefault();
        this.focusScenarioLinkElement(0);
    };
    AppComponent.prototype.onScenarioLinkKeyDown = function (scenarioElement, filterElement, event) {
        event.preventDefault();
        switch (event.key) {
            case 'ArrowUp':
                this.goUp(scenarioElement);
                break;
            case 'ArrowDown':
                this.goDown(scenarioElement);
                break;
            default:
                if (event.key !== 'Escape' && event.key !== 'Enter') {
                    if (event.key.length === 1) {
                        this.filter.setValue("" + this.filter.value + event.key);
                    }
                    filterElement.focus();
                }
                break;
        }
    };
    AppComponent.prototype.onScenarioLinkKeyUp = function (scenarioElement, event) {
        event.preventDefault();
        switch (event.key) {
            case 'Escape':
                this.commandBarActive = false;
                break;
            case 'Enter':
                scenarioElement.click();
                break;
        }
    };
    AppComponent.prototype.onScenarioClick = function (sandboxKey, scenarioKey, e) {
        this.selectScenario(sandboxKey, scenarioKey);
        e.preventDefault();
    };
    AppComponent.prototype.isSelected = function (sandbox, scenario) {
        return this.selectedSandboxAndScenarioKeys.scenarioKey === scenario.key
            && this.selectedSandboxAndScenarioKeys.sandboxKey.toLowerCase() === sandbox.key.toLowerCase();
    };
    AppComponent.prototype.goUp = function (scenarioElement) {
        var currentIndex = -1;
        this.scenarioLinkElements.find(function (scenarioElementRef, index) {
            if (scenarioElementRef.nativeElement === scenarioElement) {
                currentIndex = index;
            }
        });
        if (currentIndex === 0) {
            this.focusScenarioLinkElement(this.scenarioLinkElements.length - 1);
        }
        else {
            this.focusScenarioLinkElement(currentIndex + -1);
        }
    };
    AppComponent.prototype.goDown = function (scenarioElement) {
        var currentIndex = -1;
        this.scenarioLinkElements.find(function (scenarioElementRef, index) {
            if (scenarioElementRef.nativeElement === scenarioElement) {
                currentIndex = index;
            }
        });
        if (currentIndex === this.scenarioLinkElements.length - 1) {
            this.focusScenarioLinkElement(0);
        }
        else {
            this.focusScenarioLinkElement(currentIndex + 1);
        }
    };
    AppComponent.prototype.focusScenarioLinkElement = function (index) {
        if (this.scenarioLinkElements.toArray()[index]) {
            this.scenarioLinkElements.toArray()[index].nativeElement.focus();
        }
    };
    AppComponent.prototype.filterSandboxes = function (sandboxes, filter) {
        if (!filter) {
            return [];
        }
        var tabIndex = 0;
        var filterNormalized = filter.toLowerCase();
        return sandboxes
            .filter(function (sandbox) { return fuzzySearch(filterNormalized, sandbox.key.toLowerCase()); })
            .sort(function (a, b) {
            var nameA = a.name.toUpperCase();
            var nameB = b.name.toUpperCase();
            if (nameA < nameB) {
                return -1;
            }
            if (nameA > nameB) {
                return 1;
            }
            return 0;
        })
            .map(function (sandbox) { return Object.assign({}, sandbox, { tabIndex: tabIndex++ }); });
    };
    AppComponent.prototype.toggleCommandBar = function () {
        this.commandBarActive = !this.commandBarActive;
    };
    AppComponent.prototype.selectScenario = function (sandboxKey, scenarioKey) {
        this.selectedSandboxAndScenarioKeys = { sandboxKey: sandboxKey, scenarioKey: scenarioKey };
        this.urlService.setSelected(sandboxKey, scenarioKey);
    };
    __decorate([
        ViewChildren('scenarioElement'), 
        __metadata('design:type', Object)
    ], AppComponent.prototype, "scenarioLinkElements", void 0);
    AppComponent = __decorate([
        Component({
            selector: 'ap-root',
            animations: [
                trigger('flyInOut', [
                    state('true', style({ transform: 'translate(-50%, 0)' })),
                    transition('void => *', [
                        style({ transform: 'translate(-50%, -110%)' }),
                        animate(100)
                    ]),
                    transition('* => void', [
                        animate(100, style({ transform: 'translate(-50%, -110%)' }))
                    ])
                ])
            ],
            styles: ["\n    :host * {\n      box-sizing: border-box;\n    }\n    :host {\n      font-family: sans-serif;\n      display: flex;\n      flex-direction: column; }\n      :host .command-bar-shield {\n        position: absolute;\n        width: 100%;\n        height: 100vh;\n        opacity: 0; }\n      :host .command-bar {\n        z-index: 1;\n        font-family: Menlo,Monaco,monospace;\n        position: absolute;\n        display: flex;\n        flex-direction: column;\n        left: 50%;\n      }\n      :host .command-bar > input {\n        font-family: Menlo,Monaco,monospace;\n        width: 365px;\n        z-index: 1;\n        padding: 4px;\n        margin: 6px 0 0 5px;\n        border: 1px solid #174a6c;\n        background-color: #3c3c3c;\n        font-size: 14pt;\n        color: #fff;\n      }\n      :host .command-bar > input::-webkit-input-placeholder {\n        color: #a9a9a9; }\n      :host .command-bar > input::-moz-focus-inner {\n        border: 0;\n        padding: 0; }\n      :host .command-bar > div {\n        margin-top: -39px;\n        width: 376px;\n        padding: 34px 0 11px 0;\n        background-color: #252526;\n        color: #fff;\n        box-shadow: 0 3px 8px 5px black;\n      }\n      :host .command-bar > div > div:first-child {\n        margin-top: 12px;\n        padding-top: 4px;\n        border-top: 1px solid black;\n      }\n      :host .command-bar > div a {\n        cursor: pointer; \n        display: block;\n        width: 100%;\n      }\n      :host .command-bar > div a:hover,\n      :host .command-bar > div a:focus {\n        background-color: #0097fb;\n        color: #fff;\n        outline-style: none;\n      }\n      :host .command-bar .sandbox {\n        font-style: italic;\n        color: rgba(255, 255, 255, .5);\n        padding: 2px 8px;\n      }\n      :host .command-bar .scenarios {\n        display: flex;\n      }\n      :host .command-bar .scenario {\n        padding: 2px 3px;\n        margin: 0 5px;\n      }\n      :host .command-bar .scenario-icon {\n        display: inline-block;\n        width: 20px;\n        height: 20px;\n        fill: #fff;\n        opacity: .2;\n        margin: 4px 0 0 2px;\n      }\n      :host .command-bar .scenario-icon.selected {\n        opacity: 1;\n        fill: #0097fb;\n      }\n       \n      :host section {\n        border: 0;\n        width: 100%;\n        background-color: white; }\n        \n      :host section.help-message {\n        display: flex;\n        align-items: center;\n        justify-content: center; \n        height: 100vh; }\n      :host section.help-message > div {\n        max-width: 50%;\n        font-family: Menlo,Monaco,monospace; }\n  "],
            template: "\n    <svg xmlns=\"http://www.w3.org/2000/svg\" style=\"display: none;\">\n      <symbol id=\"icon-pin\" viewBox=\"25 25 50 50\">\n        <path d=\"M70.32,34.393l-7.628-8.203c-0.854-0.916-2.256-1.066-3.281-0.342l-13.699,9.639c-0.479-0.055-0.956-0.082-1.425-0.082 c-5.935,0-9.126,4.326-9.259,4.51c-0.718,0.994-0.612,2.359,0.249,3.232l7.88,7.98L30.436,63.848c-0.98,0.98-0.98,2.568,0,3.549 c0.49,0.49,1.132,0.734,1.774,0.734c0.642,0,1.284-0.244,1.773-0.734l12.7-12.699l7.34,7.432c0.484,0.49,1.131,0.746,1.786,0.746 c0.436,0,0.874-0.113,1.27-0.346c4.014-2.357,3.876-9.373,3.557-12.727l9.799-12.125C71.22,36.707,71.171,35.307,70.32,34.393z M56.073,47.465c-0.432,0.535-0.626,1.225-0.536,1.906c0.332,2.51,0.239,5.236-0.146,7.002L40.678,41.475 c0.868-0.551,2.079-1.051,3.61-1.051c0.5,0,1.02,0.053,1.546,0.158c0.674,0.137,1.375-0.01,1.938-0.408l12.737-8.963l4.655,5.006 L56.073,47.465z\"></path>\n      </symbol>\n    </svg>\n    <div class=\"command-bar-shield\" *ngIf=\"commandBarActive\" (click)=\"toggleCommandBar()\"></div>\n    <div class=\"command-bar\" *ngIf=\"commandBarActive\" [@flyInOut]=\"commandBarActive\">\n      <input type=\"text\" name=\"filter\" placeholder=\"filter\" \n        [formControl]=\"filter\" \n        #filterElement\n        [apFocus]=\"commandBarActive\"\n        (keyup.Esc)=\"commandBarActive = false\"\n        (keydown.ArrowDown)=\"goToFirstScenario($event)\">\n      <div>\n        <div *ngFor=\"let sandbox of filteredSandboxes\">\n          <div class=\"sandbox\"\n               [class.selected]=\"selectedSandboxAndScenarioKeys.sandboxKey === sandbox.key\">\n            {{sandbox.prependText}}{{sandbox.name}}</div>\n          <div *ngFor=\"let scenario of sandbox.scenarios\" class=\"scenarios\">\n            <svg class=\"scenario-icon\" [class.selected]=\"isSelected(sandbox, scenario)\">\n              <use xlink:href=\"#icon-pin\"/>\n            </svg>\n            <a\n               class=\"scenario\"\n               #scenarioElement\n               [tabindex]=\"scenario.tabIndex\"\n               (keydown)=\"onScenarioLinkKeyDown(scenarioElement, filterElement, $event)\"\n               (keyup)=\"onScenarioLinkKeyUp(scenarioElement, $event)\"\n               (click)=\"onScenarioClick(sandbox.key, scenario.key, $event); toggleCommandBar()\"\n               [class.selected]=\"isSelected(sandbox, scenario)\">\n              {{scenario.description}}</a>\n          </div>\n        </div>\n      </div>\n    </div>\n    <section *ngIf=\"!selectedSandboxAndScenarioKeys.sandboxKey\" class=\"help-message\">\n      <div>\n        <template [ngIf]=\"totalSandboxes > 0\">\n          <p>The app has {{totalSandboxes}} sandboxed component{{totalSandboxes > 1 ? 's' : ''}} loaded.</p>\n        </template>\n        <template [ngIf]=\"totalSandboxes === 0\">\n          <p>The app does not have any sandboxed components.</p>\n        </template>\n        <p>Pick sandboxed components: <strong>ctrl + o</strong></p>\n      </div>\n    </section>\n    <section *ngIf=\"selectedSandboxAndScenarioKeys.sandboxKey\">\n      <ap-scenario [selectedSandboxAndScenarioKeys]=\"selectedSandboxAndScenarioKeys\"></ap-scenario>\n    </section>\n  "
        }),
        __param(0, Inject(SANDBOXES)), 
        __metadata('design:paramtypes', [Array, StateService, UrlService, EventManager])
    ], AppComponent);
    return AppComponent;
}());
